

function [x, res, iter] = Subcg(L, b, tol, maxiter, initial,verbose, option)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solve a symmetric positive definite system Lx = b via CG               %
% Input:                                                                 %
%            L - Either an NxN matrix, or a function handle              %
%            b - N vector                                                %
%          tol - Desired precision.  Algorithm terminates                %
%                when norm(Ax-b) <= tol                                  %
%      maxiter - Maximum number of iterations (defaut length(b))         %
%      initial - Initial value (defaut 0)                                %
%      verbose - If 0, do not print out progress messages                %
%       option - a stucture for subset cg                                %
%             .subset    -  index of conlums of A used                   %
%             .Lt        - transposed of L                               %
%             .p         - number of whole conlums                       %
% Output:                                                                %
%            x -  soultion                                               %
%          res -  residual                                               %
%         iter -  number of iterations                                   %
% Copyright (c)  Yuling Jiao(yulingjiaomath@whu.edu.cn)                  %
% Created on 17 October, 2013                                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


if ~exist('L','var')|| ~exist('b','var')
    disp('error, not enough inputment');
end
implicit = isa(L,'function_handle'); 
if ~exist('verbose','var')
    verbose = 0; 
end
if nargin < 5
    initial = zeros(size(b));
end
if nargin < 4
    maxiter = length(b);
end
if nargin < 4
    tol = 1e-5;
end
if ~exist('option','var')||isempty(option)
    %disp('Full CG')
    Aop = L;
    x = initial;
else
    %disp('Subset CG')
    if (implicit)
      subset = option.subset;
      p = option.p;
      Lt = option.Lt;
      Ssubt = @(z) upsam(z,subset,p);
      Ssub = @(z) z(subset,:);
      Aop = @(z) Ssub(Lt(L(Ssubt(z))));
      x = Ssub(initial); 
    else
      subset = option.subset;
      Lt = option.Lt;
      Ssub = @(z) z(subset,:);
      Aop = @(z) Lt*(L*(z));
      x = Ssub(initial); 
      implicit = 1;
    end
    
end

if (implicit)
    r = b - Aop(x); 
else
    r = b - Aop*x; 
end
d = r;
delta = r'*r;
iter = 0;
while ((iter < maxiter) && (delta > tol^2))
  if (implicit), q = Aop(d);  else  q = Aop*d;  end
 
  Alpha = delta/(d'*q);
  x = x + Alpha*d;
  r = r - Alpha*q;
  deltaold = delta;
  delta = r'*r;
  beta = delta/deltaold;
  d = r + beta*d;
  iter = iter + 1;
end
  res = sqrt(delta);
if (verbose)
  disp(sprintf('cg: Iterations = %d, best residual = %14.8e', numiter, bestres));
end
end
%% subfunctions
function upz = upsam(z,id,nn)
upz = zeros(nn,1);
upz(id) = z;
end