function [x,lam,ithist] = pdascl1(X,Xt,y,opts)
%-----------------------------------------------------------------------%
% minimize 1/2||X*x-y||^2  + lambda ||x||_1 +  alpha/2||x||^2           %
% by PDASC-l1 algorithm with automatic regualrization parameter \lambda %
% selection rule : DP, MDP,BIC on path   Lam ={lam_{1},...,lam_{N}      %
% (c) by Yuling Jiao (yulingjiaomath@whu.edu.cn)                        %
% Created on Oct 17, 2013                                               %
%                                                                       %
% INPUTS:                                                               %
%     X  ---  sampling matrix (normalized)                              %
%     Xt ---  transpose of sampling matrix                              %
%     y  ---  data vector                                               %
%   opts ---  structure containing                                      %
%       alpha --- regularization parameter (defaut: 0)                  %
%         N   --- length of path (defaut: 100)                          %
%       Lmin  --- minimun in Lam (defaut: 1e-10)                        %
%        del  --- noise level                                           %
%        mu   --- stop if ||x_{lambda_{k}}||> opts.mu*n (defaut: 0.5)   %
%        x0   --- initial value for PDASC-l1 (defaut: 0)                %
%     MaxIter --- maximum number of iterations in PDAS  (default: 1)    %
%    debias   --- 1 get the result after debias, 0 not (default:1)      %
% OUTPUTS:                                                              %
%     x   ---- reconstructed signal                                     %
%    lam  ---- regu. parameter (by discrepancy principle or BIC)        %
%  ithist ---- structure on iteration history, containing               %
%          .x  --- solution path                                        %
%          .as --- size of active set                                   %
%          .it --- # of iteration on the path                           %
%          .res--- residual on the path                                 %
% ======================================================================%

implicit = isa(X,'function_handle');
if implicit == 0
   [n,p] = size(X);
   Xty = Xt*y;
else
   n = opts.n;
   p = opts.p;
   Xty = Xt(y);
end
linf  = norm(Xty,inf);
x0    = zeros(p,1);
if ~exist('opts','var')
    opts.alpha = 0;
    opts.N     = 100;
    opts.Lmax  = 1;
    opts.Lmin  = 1e-10;
    opts.MaxIt = 1;
    opts.mu    = 0.5;
    opts.del = norm(y-ye);
    opts.debias = 1;
end
opts.Xty  = Xty;
normy = norm(y)^2;
opts.normy = normy;
% construct the continuation path
Lam   = exp(linspace(log(opts.Lmax),log(opts.Lmin),opts.N))';
Lam   = Lam(2:end);
Lam = Lam*linf;
ithist.Lam = Lam;
%% main loop for pathfolling and choosing lambda and output solution
ithist.x   = [];
ithist.as  = [];
ithist.it  = [];
ithist.res = [];
ithist.bic = [];

for k = 1:length(Lam)
    opts.lam = Lam(k);
    [x,s,A,xdb,dbyA,hiscg] = pdasl1(X,Xt,y,opts,x0);
    x0 = x;
    ithist.x(:,k)= x;
    ithist.as = [ithist.as;s];    % size of active set
%------------- \|X(x+(X(:,A)'*X(:,A))\d(A))-y\|----------------------------%    
      res = sqrt(abs(normy + dbyA'*xdb(A))); % DP and MDP
%-------------------------------------------------------------------------%
    ithist.res = [ithist.res; res];
    ithist.bic= [ithist.bic; 0.5*res^2 + log(n)*s/(n)];  %  Bayesian inf. criterion
    if res <= opts.del
      if opts.debias == 0
        disp('Discrepancy principle is satisfied')
      else
          disp('Modified Discrepancy principle is satisfied')
      end
      lam = Lam(k);
      x(A)   = xdb(A);
       break
    end
    if s > opts.mu*n
        display('Too many nonzeros in the reconstruction ....')
          break
    end      
end
 if res > opts.del
            % Bayesian information criterion
             disp(' BIC is used ')
             ii = find(ithist.bic == min(ithist.bic));
             ii = ii(end);
             x  = ithist.x(:,ii);
             lam= Lam(ii);
 end
